#Content Insights Android SDK

## About
Content Insights Android SDK helps you integrate CI tracking mechanism in your Android mobile application. 

There are three server calls in this SDK:

* **Article Load** - Indicates that article is loaded on the screen. 
* **Article Read** - Indicates that user spent at least 10s with the article on the screen. This is considered by the system as a signal that user started reading the article. This call is very similar to the previous one (apart from few parameters) and it's fired 10s after the first one. 
* **Attention time** -  This call is triggered the first time when the article is opened and continue to be triggered every 5 seconds while the user is on the article. We start tracking attention time right from he start so we don't miss first 10s. 

## Installation

To install Content Insights SDK into your application, include ```ContentInsights-X.Y.aar``` as a dependency using Gradle.  
```X.Y``` in the file name stands for SDK major and minor version. 

One of the ways of doing this is putting the file into your project's ```libs``` folder and telling
Gradle to use it. In ```build.gradle``` Android Studio usually creates ```implementation fileTree(include: ['*.jar'], dir: 'libs')```
line in order to include all ```.jar``` dependencies in ```lib``` folder. To include ```.aar``` file just edit that line
to look like this: ```implementation fileTree(include: ['*.jar', '*.aar'], dir: 'libs')```

## Initialization

Before usage, SDK should be initialized in the project's ```Application``` class.

Typical initialization looks like this:

```
ContentInsightsConfig config =
        new ContentInsightsConfig.Builder(this, "domain_id")
                .build();
ContentInsights.init(config);
```

All configuration of SDK behavior is done using ```ContentInsightsConfig``` instance.
For example, the following configuration sets SDK not to send real API calls and instead log requests under ```ContentInsights``` tag.

```
ContentInsightsConfig config =
       new ContentInsightsConfig.Builder(this, "domain_id")
               .noBackendRequests()
               .doLogWithTag("ContentInsights")
               .build();
ContentInsights.init(config);
```

## Tracking Article Load

Loading article means that article page/screen has been opened to be read.
To track this event set desired parameters and request tracking to be done by the SDK.

```
ArticleReadTracker articleReadTracker = ContentInsights.tracking().startArticleReadTracker(articleUrl, articleId);
```

## Tracking Article Read

Tracking Article Read will be done automatically by the SDK, 10s after ```startArticleReadTracker``` has been called.

## Tracking Attention Time

For attention time there is ```AttentionTimeTracker``` that can be obtained from SDK in a similar way
as ```ArticleReadTracker```.

```
AttentionTimeTracker attentionTracker = ContentInsights.tracking().startAttentionTracker(articleUrl, articleId);
```

In order to achieve maximum flexibility with older Android SDK levels, tracking when the app/screen has user's attention is done manually.

Communication regarding when app gets or loses user's attention is done by calling ```ContentInsights.tracking().startAttention()``` 
and ```ContentInsights.tracking().pauseAttention()```

Our suggestion is to use ```Activity``` lifecycle methods for doing these calls, but it's up to the specific implementation to find the most suitable model. 

For example:

```
@Override
public void onStart() {
    super.onStart();
    ContentInsights.tracking().startAttention();
}

@Override
public void onPause() {
    super.onPause();
    ContentInsights.tracking().pauseAttention();
}
```

Apart from notifying when the article has user attention or not, no further calls are required in order to track attention.

## Tracking Scrolling Activity

As part of ```AttentionTimeTracker```, there is an option to track how much user has scrolled. In
order to have valid statistics, there is a formula for calculating scrolling:
```((viewportHeight + scrollTop) * 100) / scrollHeight```

New scroll position is marked manually by calling ```ContentInsights.tracking().setNewScrollPosition(newPos)```
whenever scroll event occurs.

Beside manual call, there are two alternative ways to track scrolls that might be used instead
of this one.

### ```ScrollView``` tracking on Android SDK level 23 and above

For articles that are served inside of ```ScrollView``` or its subclass, there is a way
to track scrolling automatically on Android SDK levels 23 and above. If that is the case,
the only call that should be made is ```ContentInsights.tracking().trackScrolling(ScrollView)```, and SDK will
detect all scrolls and calculate them according to the formula.

### ```WebView``` tracking with accompanying JavaScript

If your articles are served as HTML via WebView, the easiest way to track
their scroll positions is by simply calling ```ContentInsights.tracking().trackScrolling(articleWebView)```

For this to work, ```WebView``` must have JavaScript enabled, and inside
of article's HTML there should be JavaScript code that tracks scrolls, does the calculation
according to the formula and does ***exactly*** this call: ```AndroidApi.setNewScrollPosition(scrollPosition)```

If you use the same HTML source of an article outside of WebView, the best practice is to use the following code
and avoid any errors in JavaScript outside of the WebView environment. 

```
if (typeof AndroidApi !== 'undefined') {
    AndroidApi.setNewScrollPosition(scrollPosition);
}
```

## Flushing the Article

When a user leaves the article, tracker mush flush the article. 
This is a sign for the tracker to finish the tracking of attention time and to do any final calls to Content Insight's servers. 
When everything is finished, the tracker is destroyed. 

To flush the article, please call ```ContentInsights.tracking().flushArticle()```


## Example application

You may review an example application located in folder **CiSDKExample**
An example app shows loading Feed of articles from the website and includes all the SDK calls mentioned above.
It shows also all the optional parameters that might be needed for specific application setups.

We strongly advise you to check out example application before implementing SDK in your application. 
