package com.contentinsights.sdk;

import com.contentinsights.sdk.helpers.Constants;
import com.contentinsights.sdk.helpers.Preferences;
import com.contentinsights.sdk.helpers.PreferencesImpl;
import com.contentinsights.sdk.helpers.RandomGenerator;
import com.contentinsights.sdk.helpers.TimeProvider;
import com.contentinsights.sdk.helpers.TokenProvider;
import com.contentinsights.sdk.api.ContentInsightsSdk;
import com.contentinsights.sdk.impl.NullSdk;
import com.contentinsights.sdk.api.SdkFeatures;
import com.contentinsights.sdk.impl.WorkingSdk;
import com.contentinsights.sdk.requests.ApiTrackingRequest;
import com.contentinsights.sdk.requests.CompositeTrackingRequest;
import com.contentinsights.sdk.requests.LoggingTrackingRequest;
import com.contentinsights.sdk.requests.TrackingRequest;

/**
 * Entry point class for using Content Insights SDK. It is used for initializing it and accessing
 * tracking features.
 */
public class ContentInsights {

    private static ContentInsightsSdk SDK;
    private static boolean IS_ALREADY_INITIALIZED = false;

    private ContentInsights() {}

    /**
     * Initialization of Content Insights SDK. This is the very first thing that needs to be done in order
     * to access SDK features. Generally, this should be called in you app's
     * {@link android.app.Application Application} implementation. It may not be called more than once.
     * @param config Configuration for initializing SDK. It may not be null. See also {@link ContentInsightsConfig}
     */
    public static void init(ContentInsightsConfig config) {
        if (IS_ALREADY_INITIALIZED) {
            return;
        }

        IS_ALREADY_INITIALIZED = true;

        int requiredSdkFeatures = config.getConfiguredSdkFeatures();

        if (requiredSdkFeatures == SdkFeatures.NOTHING) {
            SDK = new NullSdk();
            return;
        }

        TimeProvider timeProvider = new TimeProvider();
        Preferences preferences = new PreferencesImpl(config.getContext().getApplicationContext());
        TokenProvider tokenProvider = new TokenProvider(preferences, new RandomGenerator(), config.getSessionTimeoutInSeconds());

        if (requiredSdkFeatures == SdkFeatures.API_AND_LOGGING) {
            initAllFeaturesSdk(timeProvider, tokenProvider, config);
        } else if (requiredSdkFeatures == SdkFeatures.LOGGING) {
            initLoggingSdk(timeProvider, tokenProvider, config);
        } else if (requiredSdkFeatures == SdkFeatures.API_CALLING){
            initApiCallingSdk(timeProvider, tokenProvider, config);
        } else {
            throw new ContentInsightsException("This configuration should be impossible");
        }
    }

    /**
     * This gives user the access to SDK features,
     * as implementation of {@link ContentInsightsSdk} interface.
     * <p>
     * Before calling this method, {@link #init(ContentInsightsConfig)} <i>must</i> be called.
     *
     * @return SDK instance.
     */
    public static ContentInsightsSdk tracking() {
        if (SDK == null) {
            throw new ContentInsightsException("You should initialize the ContentInsights SDK first.");
        }
        return SDK;
    }

    private static void initAllFeaturesSdk(TimeProvider timeProvider,
                                           TokenProvider tokenProvider, ContentInsightsConfig config) {
        String tag = config.getLogTag();
        String url = config.getApiUrl();

        LoggingTrackingRequest logRead = new LoggingTrackingRequest(tag, url + Constants.API_READ_ENDPOINT);
        LoggingTrackingRequest logAttention = new LoggingTrackingRequest(tag, url + Constants.API_ATTENTION_ENDPOINT);
        ApiTrackingRequest apiRead = new ApiTrackingRequest(url + Constants.API_READ_ENDPOINT);
        ApiTrackingRequest apiAttention = new ApiTrackingRequest(url + Constants.API_ATTENTION_ENDPOINT);
        TrackingRequest readTrackingRequest = new CompositeTrackingRequest(logRead, apiRead);
        TrackingRequest attentionTrackingRequest = new CompositeTrackingRequest(logAttention, apiAttention);
        SDK = new WorkingSdk(config.getDomainId(), timeProvider, tokenProvider,
                readTrackingRequest, attentionTrackingRequest);
    }

    private static void initLoggingSdk(TimeProvider timeProvider,
                                       TokenProvider tokenProvider, ContentInsightsConfig config) {

        String tag = config.getLogTag();
        String url = config.getApiUrl();
        TrackingRequest readTrackingRequest = new LoggingTrackingRequest(tag, url + Constants.API_READ_ENDPOINT);
        TrackingRequest attentionTrackingRequest = new LoggingTrackingRequest(tag, url + Constants.API_ATTENTION_ENDPOINT);
        SDK = new WorkingSdk(config.getDomainId(), timeProvider, tokenProvider,
                readTrackingRequest, attentionTrackingRequest);
    }

    private static void initApiCallingSdk(TimeProvider timeProvider,
                                       TokenProvider tokenProvider, ContentInsightsConfig config) {

        String apiUrl = config.getApiUrl();
        TrackingRequest readTrackingRequest = new ApiTrackingRequest(apiUrl + Constants.API_READ_ENDPOINT);
        TrackingRequest attentionTrackingRequest = new ApiTrackingRequest(apiUrl + Constants.API_ATTENTION_ENDPOINT);
        SDK = new WorkingSdk(config.getDomainId(), timeProvider, tokenProvider,
                readTrackingRequest, attentionTrackingRequest);
    }
}
