package com.contentinsights.sdk;

import android.content.Context;

import com.contentinsights.sdk.helpers.Constants;
import com.contentinsights.sdk.api.SdkFeatures;

/**
 * Configuration class for initializing SDK. See also {@link ContentInsights#init(ContentInsightsConfig)}
 *
 * Instead of using regular constructor, it is being build by {@link ContentInsightsConfig.Builder}
 */
public class ContentInsightsConfig {

    /**
     * Default session timeout, 1800 seconds.
     */
    public static int DEFAULT_TIMEOUT = 1800;

    /**
     * Default logging tag, "ContentInsights"
     */
    public static String DEFAULT_LOG_TAG = "ContentInsights";

    private Context context;
    private String domainId;
    private int sessionTimeout;
    private boolean shouldLog;
    private boolean isDebug;
    private String apiUrl;
    private String logTag;

    private ContentInsightsConfig(Context context, String domainId, int sessionTimeout,
                                  boolean shouldLog, boolean isDebug,
                                  String apiUrlRoot, String logTag) {
        this.domainId = domainId;
        this.sessionTimeout = sessionTimeout;
        this.shouldLog = shouldLog;
        this.isDebug = isDebug;
        this.apiUrl = apiUrlRoot;
        this.logTag = logTag;
        this.context = context;
    }

    /**
     * @return Domain ID
     */
    public String getDomainId() {
        return domainId;
    }

    /**
     * @return Session timeout in seconds. It represents timeout using for assuming
     * whether user has started new session of browsing the content in app or not.
     */
    public int getSessionTimeoutInSeconds() {
        return sessionTimeout;
    }

    /**
     *
     * @return Representation of backend api url for recording tracking data.
     */
    public String getApiUrl() {
        return apiUrl;
    }

    /**
     *
     * @return Tag that's being used when configuration is set to log tracking requests.
     */
    public String getLogTag() {
        return logTag;
    }

    /**
     *
     * @return Number representation of configured SDK features. See also {@link SdkFeatures}
     */
    public int getConfiguredSdkFeatures() {
        if (!isDebug && shouldLog) {
            return SdkFeatures.API_AND_LOGGING;
        }
        if (shouldLog) {
            return SdkFeatures.LOGGING;
        }
        if (isDebug) {
            return SdkFeatures.NOTHING;
        }
        return SdkFeatures.API_CALLING;
    }

    /**
     *
     * @return {@link Context}
     */
    public Context getContext() {
        return context;
    }

    /**
     * Builder pattern implementation for constructing {@link ContentInsightsConfig}
     */
    public static class Builder {

        private Context context;
        private String domainId;
        private int sessionTimeout = DEFAULT_TIMEOUT;
        private String apiUrl = Constants.API_ROOT;
        private boolean shouldLog = false;
        private boolean isDebug = false;
        private String logTag = DEFAULT_LOG_TAG;

        /**
         *
         * @param context {@link Context}. Null not allowed.
         * @param domainId Domain ID. Null or empty string not allowed.
         */
        public Builder(Context context, String domainId) {
            if (context == null || domainId == null || domainId.isEmpty()) {
                throw new ContentInsightsException(ContentInsightsException.BAD_DATA);
            }

            this.context = context;
            this.domainId = domainId;
        }

        /**
         *
         * @param context {@link Context}. Null not allowed.
         * @param domainId Domain ID.
         */
        public Builder(Context context, int domainId) {
            this(context, "" + domainId);
        }

        /**
         * Setting custom api url. Default value points to default production backend.
         *
         * @param apiUrl Custom api url. Null or empty string now allowed.
         * @return This {@link Builder} instance.
         */
        public Builder customApiUrl(String apiUrl) {
            if (apiUrl == null || apiUrl.isEmpty()) {
                throw new ContentInsightsException(ContentInsightsException.BAD_DATA);
            }
            this.isDebug = false;
            this.apiUrl = apiUrl;
            return this;
        }

        /**
         * Sets configuration in a way that no backend requests should be made.
         *
         * @return This {@link Builder} instance.
         */
        public Builder noBackendRequests() {
            this.isDebug = true;
            return this;
        }

        /**
         * Setting custom session timeout. Default value is {@link ContentInsightsConfig#DEFAULT_TIMEOUT}.
         *
         * @param timeout Desired timeout in seconds. Must be positive number.
         * @return This {@link Builder} instance.
         */
        public Builder sessionTimeoutInSeconds(int timeout) {
            if (timeout < 0) {
                throw new ContentInsightsException(ContentInsightsException.BAD_DATA);
            }
            this.sessionTimeout = timeout;
            return this;
        }

        /**
         * Sets configuration in a way that all tracking requests get logged.
         * Tag for logging that's going to be used is {@link ContentInsightsConfig#DEFAULT_LOG_TAG}
         *
         * @return This {@link Builder} instance.
         */
        public Builder doLog() {
            this.shouldLog = true;
            return this;
        }

        /**
         * Sets configuration in a way that all tracking requests get logged.
         * Also, lets user to put desired logging tag.
         *
         * @param logTag Desired logging tag. Null or empty string not allowed.
         * @return This {@link Builder} instance.
         */
        public Builder doLogWithTag(String logTag) {
            if (logTag == null || logTag.isEmpty()) {
                throw new ContentInsightsException(ContentInsightsException.BAD_DATA);
            }
            this.shouldLog = true;
            this.logTag = logTag;
            return this;
        }

        /**
         * Finalizer method that constructs {@link ContentInsightsConfig} instance.
         *
         * @return {@link ContentInsightsConfig} instance.
         */
        public ContentInsightsConfig build() {
            return new ContentInsightsConfig(this.context, this.domainId, this.sessionTimeout,
                    this.shouldLog, this.isDebug, this.apiUrl, this.logTag);
        }
    }
}
