package com.contentinsights.sdk.api;

import com.contentinsights.sdk.ContentInsightsException;
import com.contentinsights.sdk.helpers.StringJoiner;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

public class ArticleReadParams {

    private static DateFormat FORMATTER = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    private String domainId;

    private List<String> authors = new LinkedList<>();
    private String title;
    private List<String> sections = new LinkedList<>();
    private List<String> tags = new LinkedList<>();
    private int numberOfComments;
    private String accessLevel;
    private String articleType;
    private String articleUrl;
    private String articleId;
    private String referrer;
    private String readerType;

    private Map<String, Integer> socialNetworkCounts = new HashMap<>();

    private int    wordCount = 0;
    private String publishedDate = "";


    public ArticleReadParams(String domainId, String articleUrl) {
        if (domainId == null || domainId.isEmpty()) {
            throw new ContentInsightsException(ContentInsightsException.BAD_DATA);
        }
        if (articleUrl == null || articleUrl.isEmpty()) {
            throw new ContentInsightsException(ContentInsightsException.BAD_DATA);
        }
        this.domainId = domainId;
        this.articleUrl = articleUrl;
    }

    public boolean hasRequiredData() {
        if (domainId == null || domainId.isEmpty()) {
            return false;
        }

        if (articleUrl == null || articleUrl.isEmpty()) {
            return false;
        }

        return true;
    }

    public ArticleReadParams setAuthors(String... authors) {
        this.authors = Arrays.asList(authors);
        return this;
    }

    public ArticleReadParams setAuthors(List<String> authors) {
        this.authors = authors;
        return this;
    }

    public ArticleReadParams addAuthor(String author) {
        this.authors.add(author);
        return this;
    }

    public ArticleReadParams setSections(String... sections) {
        this.sections = Arrays.asList(sections);
        return this;
    }

    public ArticleReadParams setSections(List<String> sections) {
        this.sections = sections;
        return this;
    }

    public ArticleReadParams addSection(String section) {
        this.sections.add(section);
        return this;
    }

    public ArticleReadParams setTitle(String title) {
        this.title = title;
        return this;
    }

    public ArticleReadParams setArticleId(String articleId) {
        this.articleId = articleId;
        return this;
    }

    public ArticleReadParams setPublishedDate(Date date) {
        this.publishedDate = FORMATTER.format(date);
        return this;
    }

    public ArticleReadParams setReferrer(String referrer) {
        this.referrer = referrer;
        return this;
    }

    public ArticleReadParams setTags(String... tags) {
        this.tags = Arrays.asList(tags);
        return this;
    }

    public ArticleReadParams setTags(List<String> tags) {
        this.tags = tags;
        return this;
    }

    public ArticleReadParams addTag(String tag) {
        this.tags.add(tag);
        return this;
    }

    public ArticleReadParams setNumberOfComments(int numberOfComments) {
        this.numberOfComments = numberOfComments;
        return this;
    }

    public ArticleReadParams setAccessLevel(String accessLevel) {
        this.accessLevel = accessLevel;
        return this;
    }

    public ArticleReadParams setArticleType(String articleType) {
        this.articleType = articleType;
        return this;
    }

    public ArticleReadParams setReaderType(String readerType) {
        this.readerType = readerType;
        return this;
    }

    public ArticleReadParams setWordCount(int wordCount) {
        this.wordCount = wordCount;
        return this;
    }

    public ArticleReadParams setSocialNetworkCounts(Map<String, Integer> socialNetworkCounts) {
        this.socialNetworkCounts = socialNetworkCounts;
        return this;
    }

    public String getDomainId() {
        return domainId;
    }

    public String getArticleId() {
        return articleId;
    }

    public String getArticleUrl() {
        return articleUrl;
    }

    public String getAuthors() {
        return new StringJoiner(",", authors).joined();
    }

    public String getTitle() {
        return title;
    }

    public String getSections() {
        return new StringJoiner(",", sections).joined();
    }

    public String getTags() {
        return new StringJoiner(",", tags).joined();
    }

    public String getNumberOfComments() {
        return String.valueOf(numberOfComments);
    }

    public String getAccessLevel() {
        return accessLevel;
    }

    public String getArticleType() {
        return articleType;
    }

    public String getReaderType() {
        return readerType;
    }

    public Map<String, Integer> getSocialNetworkCounts() {
        return socialNetworkCounts;
    }

    public String getWordCount() {
        return String.valueOf(wordCount);
    }

    public String getPublishedDate() {
        return publishedDate;
    }

    public String getReferrer() {
        return referrer;
    }
}
