package com.contentinsights.sdk.api;

import android.app.Activity;
import android.support.annotation.RequiresApi;
import android.webkit.WebView;
import android.widget.ScrollView;

/**
 * Attention time tracker. It tracks time and activity spent on article.
 */
public interface AttentionTimeTracker {

    /**
     * Method for getting instance of {@link AttentionTimeParams}
     * @return Attention time parameters that should be tracked. See also {@link AttentionTimeParams}
     */
    AttentionTimeParams getParams();

    /**
     * Initializes tracker time when article has an attention. It is up to implementing application
     * developers to decide what does it mean. Generally, {@link Activity#onStart()} or
     * {@link Activity#onResume()} are good candidates, but feel free to use whatever logic you find appropriate.
     */
    void initializeTracker();

    // TODO This method should not be visible to clients. Sort this out somehow.
    void submitAttentionTime();

    /**
     * Marks new scrolling position. In order to have proper statistics, position has to be calculated
     * using following formula:
     * <p>
     * <code>(viewportHeight + scrollTop) * 100) / scrollHeight</code>
     * <p>
     * There are two alternative helper methods that might be used instead of this one. <i>Never</i>
     * use more than one method at the same time, you won't get proper results.
     * <p>
     * For Android API level 23 or higher, scrolling based on {@link ScrollView} can be tracked by using
     * {@link #trackScrolling(ScrollView)} method, which calculates new scroll position and calls
     * this method by itself.
     * <p>
     * For certain articles served as html in {@link WebView} {@link #trackScrolling(WebView)}
     * may be used.
     *
     * @param position Calculated scrolling position.
     */
    void setNewScrollPosition(int position);

    /**
     * Marks the point in time when the article gets user's attention.
     * <p>
     * It is up to implementing application developers to decide what does it mean.
     * Generally, {@link Activity#onStart()} or {@link Activity#onResume()} are good candidates, but
     * feel free to use whatever logic you find appropriate.
     */
    void startAttention();

    /**
     * Marks the point in time when the article loses user's attention. This method doesn't trigger
     * any request being made.
     * <p>
     * It is up to implementing application developers to decide what does it mean. Generally,
     * {@link Activity#onPause()} or {@link Activity#onStop()} are good candidates, but feel free to
     * use whatever logic you find appropriate.
     */
    void pauseAttention();

    /**
     * For Android API levels 23 or higher, where {@link ScrollView} is used as a scrolling article
     * mechanism, this method can be used for automatic calculating of scrolling positions and sending
     * statistics.
     * <p>
     * This is just a helper method that works only in before mentioned cases. General purpose method
     * that could be used otherwise or instead of this one is {@link #setNewScrollPosition(int)} but it
     * requires on host app developer to calculate position based on given formula.
     *
     * @param scrollView Scrolling article host
     */
    @RequiresApi(23)
    void trackScrolling(ScrollView scrollView);

    /**
     * If article is presented using {@link WebView}, this method could be used for automatic
     * calculating of scrolling positions and sending statistics.
     * <p>
     * Current implementation assumes that html contains javascript that does all calculations
     * and does the following call:
     * <p>
     * <code>AndroidApi.setNewScrollPosition(newScrollPosition);</code>
     *
     * @param webView Scrolling html article host
     */
    void trackScrolling(WebView webView);

}
