package com.contentinsights.sdk.api;

import android.app.Activity;
import android.support.annotation.Nullable;
import android.support.annotation.RequiresApi;
import android.webkit.WebView;
import android.widget.ScrollView;

/**
 * Interface for accessing SDK features.
 */
public interface ContentInsightsSdk {

    /**
     * Method for starting {@link AttentionTimeTracker} tracker.
     * @param articleUrl url of article that needs to be tracked.
     * @param articleId Post id of article that needs to be tracked
     * @param referrerUrl url of article that needs to be tracked.
     * @return {@link AttentionTimeTracker}
     */
    AttentionTimeTracker startAttentionTracker(String articleUrl, String articleId, @Nullable String referrerUrl);

    /**
     * Method for starting {@link AttentionTimeTracker} tracker.
     * @param articleUrl url of article that needs to be tracked.
     * @param articleId Post id of article that needs to be tracked
     * @return {@link AttentionTimeTracker}
     */
    AttentionTimeTracker startAttentionTracker(String articleUrl, String articleId);

    /**
     * Method for starting {@link ArticleReadTracker} tracker.
     * @param articleUrl url of article that needs to be tracked.
     * @param articleId Post id of article that needs to be tracked
     * @param referrerUrl url of article that needs to be tracked.
     * @return {@link ArticleReadTracker}
     */
    ArticleReadTracker startArticleReadTracker(String articleUrl, String articleId, @Nullable String referrerUrl);

    /**
     * Method for starting {@link ArticleReadTracker} tracker.
     * @param articleUrl url of article that needs to be tracked.
     * @param articleId Post id of article that needs to be tracked
     * @return {@link ArticleReadTracker}
     */
    ArticleReadTracker startArticleReadTracker(String articleUrl, String articleId);

    /**
     * Method for flushing the article. This should be called when tracking some article is done,
     * when user is leaving the page with that article. If {@link #startArticleReadTracker(String)} or
     * {@link #startAttentionTracker(String)} called afterwards, new instance will get created.
     */
    void flushArticle();

    /**
     * Method for setting scroll position for current screen
     *
     * @param position position on the current screen
     */
    void setNewScrollPosition(int position);

    /**
     * For Android API levels 23 or higher, where {@link ScrollView} is used as a scrolling article
     * mechanism, this method can be used for automatic calculating of scrolling positions and sending
     * statistics.
     * <p>
     * This is just a helper method that works only in before mentioned cases. General purpose method
     * that could be used otherwise or instead of this one is {@link #setNewScrollPosition(int)} but it
     * requires on host app developer to calculate position based on given formula.
     *
     * @param scrollView Scrolling article host
     */
    @RequiresApi(23)
    void trackScrolling(ScrollView scrollView);

    /**
     * If article is presented using {@link WebView}, this method could be used for automatic
     * calculating of scrolling positions and sending statistics.
     * <p>
     * Current implementation assumes that html contains javascript that does all calculations
     * and does the following call:
     * <p>
     * <code>AndroidApi.setNewScrollPosition(newScrollPosition);</code>
     *
     * @param webView Scrolling html article host
     */
    void trackScrolling(WebView webView);

    /**
     * Marks the point in time when the article gets user's attention.
     * <p>
     * It is up to implementing application developers to decide what does it mean.
     * Generally, {@link Activity#onStart()} or {@link Activity#onResume()} are good candidates, but
     * feel free to use whatever logic you find appropriate.
     */
    void startAttention();

    /**
     * Marks the point in time when the article loses user's attention. This method doesn't trigger
     * any request being made.
     * <p>
     * It is up to implementing application developers to decide what does it mean. Generally,
     * {@link Activity#onPause()} or {@link Activity#onStop()} are good candidates, but feel free to
     * use whatever logic you find appropriate.
     */
    void pauseAttention();

}
