package com.contentinsights.sdk.impl;

import android.os.Handler;

import com.contentinsights.sdk.BuildConfig;
import com.contentinsights.sdk.ContentInsightsException;
import com.contentinsights.sdk.api.ArticleReadParams;
import com.contentinsights.sdk.api.ArticleReadTracker;
import com.contentinsights.sdk.helpers.Constants;
import com.contentinsights.sdk.helpers.ParametersMap;
import com.contentinsights.sdk.helpers.TokenProvider;
import com.contentinsights.sdk.requests.TrackingRequest;

import java.util.Map;

public class WorkingArticleReadTracker implements ArticleReadTracker {

    private static String INITIAL_REQUEST_TYPE_IDENTIFIER = "0";
    private static String READING_REQUEST_TYPE_IDENTIFIER = "1";

    private ArticleReadParams articleReadParams;
    private TrackingRequest request;
    private TokenProvider tokenProvider;

    private boolean pageLoaded = false;
    private int articleReadTriggerTime = 10000; // ms

    public WorkingArticleReadTracker(String articleUrl,
                                     String articleId,
                                     String domainId,
                                     TrackingRequest request,
                                     TokenProvider tokenProvider){

        if (    articleId == null || articleId.isEmpty() ||
                articleUrl == null || articleUrl.isEmpty() || domainId == null ||
                domainId.isEmpty() || request == null || tokenProvider == null
        ) {
            throw new ContentInsightsException(ContentInsightsException.BAD_DATA);
        }

        this.request = request;
        this.tokenProvider = tokenProvider;
        articleReadParams = new ArticleReadParams(domainId, articleUrl);
        articleReadParams.setArticleId(articleId);
    }

    @Override
    public ArticleReadParams getParams() {
        return articleReadParams;
    }

    public void setPageLoaded(boolean status)
    {
        this.pageLoaded = status;
    }

    @Override
    public void sendArticleLoadRequest() {
        if (!articleReadParams.hasRequiredData()) {
            throw new ContentInsightsException("Missing some required parameters. Please, check the docs");
        }

        tokenProvider.makeNewVisitToken();
        final ParametersMap requestParams = generateRequestParams();
        request.send(requestParams);

        setPageLoaded(true);

        final Handler handler = new Handler();
        handler.postDelayed(new Runnable() {
            @Override
            public void run() {
                sendArticleReadRequest(requestParams);
            }
        }, this.articleReadTriggerTime);
    }

    private void sendArticleReadRequest(ParametersMap requestParams) {
        if (!pageLoaded) {
            return;
        }

        requestParams.put("wc", String.valueOf(articleReadParams.getWordCount()));
        requestParams.put("t", READING_REQUEST_TYPE_IDENTIFIER);

        Map<String, Integer> socials = articleReadParams.getSocialNetworkCounts();
        for (String key: socials.keySet()) {
            if (socials.get(key) > 0) {
                requestParams.putPositive(key, socials.get(key));
            }
        }

        request.send(requestParams);
    }

    private ParametersMap generateRequestParams() {
        ParametersMap requestParams = new ParametersMap();
        requestParams.put("a", articleReadParams.getAuthors());
        requestParams.put("b", articleReadParams.getReferrer());
        requestParams.put("c", articleReadParams.getTitle());
        requestParams.put("d", articleReadParams.getArticleUrl());
        requestParams.put("e", articleReadParams.getSections());
        requestParams.put("f", articleReadParams.getDomainId());
        requestParams.put("g", articleReadParams.getPublishedDate());
        requestParams.put("h", articleReadParams.getTags());
        requestParams.put("i", articleReadParams.getNumberOfComments());
        requestParams.put("j", articleReadParams.getAccessLevel());
        requestParams.put("k", articleReadParams.getArticleType());
        requestParams.put("pid", articleReadParams.getArticleId());
        requestParams.put("u", tokenProvider.getSessionToken());
        requestParams.put("ul", tokenProvider.getLifetimeToken());
        requestParams.put("t", INITIAL_REQUEST_TYPE_IDENTIFIER);
        requestParams.put("x", tokenProvider.getVisitToken());
        requestParams.put("ver", String.valueOf(BuildConfig.VERSION_CODE));
        requestParams.put("ch", Constants.CHANNEL);
        return requestParams;
    }
}
