package com.contentinsights.sdk.impl;

import android.support.annotation.RequiresApi;
import android.view.View;
import android.webkit.WebView;
import android.widget.ScrollView;

import com.contentinsights.sdk.BuildConfig;
import com.contentinsights.sdk.ContentInsightsException;
import com.contentinsights.sdk.api.AttentionTimeParams;
import com.contentinsights.sdk.api.AttentionTimeTracker;
import com.contentinsights.sdk.helpers.Constants;
import com.contentinsights.sdk.helpers.ParametersMap;
import com.contentinsights.sdk.helpers.TimeProvider;
import com.contentinsights.sdk.helpers.TokenProvider;
import com.contentinsights.sdk.requests.TrackingRequest;

import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;

public class WorkingAttentionTimeTracker implements AttentionTimeTracker {

    private TrackingRequest request;
    private TokenProvider tokenProvider;
    private TimeProvider timeProvider;

    private InternalAttentionTimeParams internalAttentionTimeParams;
    private AttentionTimeParams attentionTimeParams;

    private Long lastSentAttentionTime = 0L;
    private int requestCounter;
    private Timer timer;

    public WorkingAttentionTimeTracker(String articleUrl, String articleId, String domainId, TrackingRequest request,
                                       TokenProvider tokenProvider, TimeProvider timeProvider) {

        if (
                articleUrl == null || articleUrl.isEmpty() ||
                domainId == null || domainId.isEmpty() ||
                articleId == null || articleId.isEmpty() ||
                request == null || tokenProvider == null || timeProvider == null
        ) {
            throw new ContentInsightsException(ContentInsightsException.BAD_DATA);
        }

        this.requestCounter = 0;

        this.request = request;
        this.tokenProvider = tokenProvider;
        this.timeProvider = timeProvider;

        internalAttentionTimeParams = new InternalAttentionTimeParams();
        internalAttentionTimeParams.setStartTime(timeProvider.getCurrentTime());
        internalAttentionTimeParams.setInFocus(true);
        attentionTimeParams = new AttentionTimeParams(domainId, articleUrl);
        attentionTimeParams.setArticleId(articleId);
    }

    public AttentionTimeParams getParams() {
        return attentionTimeParams;
    }

    public void initializeTracker() {
        if (timer != null) {
            return;
        }
        if (!attentionTimeParams.hasRequiredData()) {
            throw new ContentInsightsException(ContentInsightsException.BAD_DATA);
        }

        timer = new Timer();
        timer.scheduleAtFixedRate(
                new TimerTask() {
                    @Override
                    public void run() {
                        if (internalAttentionTimeParams.shouldSendAttentionRequest()) {
                            long attentionTime = timeProvider.getCurrentTime() - internalAttentionTimeParams.getStartTime();
                            internalAttentionTimeParams.setAttentionTime(attentionTime);
                            submitAttentionTime();
                            internalAttentionTimeParams.setActionMade(false);
                            lastSentAttentionTime = attentionTime;
                            requestCounter++;
                        }
                    }
                },
                0,
                Constants.ATTENTION_TIME_STEP);
    }

    public void submitAttentionTime() {
        Map<String, String> requestParams = generateRequestParams();
        request.send(requestParams);
    }

    public void setNewScrollPosition(int position) {
        internalAttentionTimeParams.setActionMade(true);
        internalAttentionTimeParams.setPosition(position);
    }

    public void startAttention()
    {
        internalAttentionTimeParams.setInFocus(true);
        internalAttentionTimeParams.setStartTime(timeProvider.getCurrentTime());
        internalAttentionTimeParams.setAttentionTimeSaved(lastSentAttentionTime);
        lastSentAttentionTime = 0L;
        initializeTracker();
    }

    public void pauseAttention()
    {
        internalAttentionTimeParams.setInFocus(false);
        if (timer != null){
            timer.cancel();
            timer = null;
        }
    }

    @Override
    @RequiresApi(23)
    public void trackScrolling(ScrollView scrollView) {
        scrollView.setOnScrollChangeListener(new View.OnScrollChangeListener() {
            @Override
            public void onScrollChange(View view, int scrollX, int scrollY, int oldScrollX, int oldScrollY) {
                float viewportHeight = view.getHeight();
                float scrollHeight = ((ScrollView)view).getChildAt(0).getHeight();
                float scrollTop = scrollY;
                int scrollPosition = Math.round(((viewportHeight + scrollTop) * 100) / scrollHeight);
                setNewScrollPosition(scrollPosition);
            }
        });
    }

    @Override
    public void trackScrolling(WebView webView) {
        webView.getSettings().setJavaScriptEnabled(true);
        webView.addJavascriptInterface(new HtmlJsTrackingInterface(new HtmlJsTrackingInterface.Callback() {
            @Override
            public void trackScrollPosition(int newScrollPosition) {
                setNewScrollPosition(newScrollPosition);
            }
        }), "AndroidApi");
    }

    private ParametersMap generateRequestParams() {
        ParametersMap requestParams = new ParametersMap();
        requestParams.put("d",   attentionTimeParams.getArticleUrl());
        requestParams.put("f",   attentionTimeParams.getDomainId());
        requestParams.put("pid", attentionTimeParams.getArticleId());
        requestParams.put("u",   tokenProvider.getSessionToken());
        requestParams.put("ul",  tokenProvider.getLifetimeToken());
        requestParams.put("at",  internalAttentionTimeParams.getAttentionTime());
        requestParams.put("ar",  internalAttentionTimeParams.getTimePassedFrom(lastSentAttentionTime));
        requestParams.put("sp",  internalAttentionTimeParams.getPosition());
        requestParams.put("ts",  timeProvider.getFormattedCurrentDate());
        requestParams.put("seq", "" + requestCounter);
        requestParams.put("x",   tokenProvider.getVisitToken());
        requestParams.put("ver", String.valueOf(BuildConfig.VERSION_CODE));
        requestParams.put("ch",  Constants.CHANNEL);
        return requestParams;
    }
}
