package com.contentinsights.sdk.impl;

import android.support.annotation.Nullable;
import android.support.annotation.RequiresApi;
import android.webkit.WebView;
import android.widget.ScrollView;

import com.contentinsights.sdk.ContentInsightsException;
import com.contentinsights.sdk.api.ArticleReadTracker;
import com.contentinsights.sdk.api.AttentionTimeTracker;
import com.contentinsights.sdk.api.ContentInsightsSdk;
import com.contentinsights.sdk.helpers.TimeProvider;
import com.contentinsights.sdk.helpers.TokenProvider;
import com.contentinsights.sdk.requests.TrackingRequest;

import java.util.Map;

public class WorkingSdk implements ContentInsightsSdk {

    private String domainId;

    private ArticleReadTracker articleReadTracker;
    private AttentionTimeTracker attentionTimeTracker;

    private TokenProvider tokenProvider;
    private TrackingRequest readTrackingRequest;
    private TrackingRequest attentionTrackingRequest;

    private TimeProvider timeProvider;

    public WorkingSdk(String domainId, TimeProvider timeProvider, TokenProvider tokenProvider,
                       TrackingRequest readTrackingRequest, TrackingRequest attentionTrackingRequest) {

        this.domainId = domainId;

        this.timeProvider = timeProvider;
        this.readTrackingRequest = readTrackingRequest;
        this.attentionTrackingRequest = attentionTrackingRequest;
        this.tokenProvider = tokenProvider;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public AttentionTimeTracker startAttentionTracker(String articleUrl, String articleId, @Nullable String referrerUrl) {
        if (articleUrl == null || articleUrl.isEmpty()) {
            throw new ContentInsightsException(ContentInsightsException.BAD_DATA);
        }

        attentionTimeTracker = new WorkingAttentionTimeTracker(
                articleUrl, articleId, domainId, attentionTrackingRequest,
                tokenProvider, timeProvider
        );

        if (referrerUrl != null) {
            attentionTimeTracker.getParams().setReferrer(referrerUrl);
        }

        attentionTimeTracker.initializeTracker();

        return attentionTimeTracker;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public AttentionTimeTracker startAttentionTracker(String articleUrl, String articleId) {
        if (articleUrl == null || articleUrl.isEmpty()) {
            throw new ContentInsightsException(ContentInsightsException.BAD_DATA);
        }

        attentionTimeTracker = new WorkingAttentionTimeTracker(
                articleUrl, articleId, domainId, attentionTrackingRequest,
                tokenProvider, timeProvider
        );

        attentionTimeTracker.initializeTracker();

        return attentionTimeTracker;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public ArticleReadTracker startArticleReadTracker(String articleUrl, String articleId, @Nullable String referrerUrl) {
        if (articleUrl == null || articleUrl.isEmpty()) {
            throw new ContentInsightsException(ContentInsightsException.BAD_DATA);
        }

        // Reset previously started trackers if exists
        flushArticle();
        articleReadTracker = new WorkingArticleReadTracker(
            articleUrl, articleId, domainId,
            readTrackingRequest, tokenProvider
        );

        if (referrerUrl != null) {
            articleReadTracker.getParams().setReferrer(referrerUrl);
        }

        articleReadTracker.sendArticleLoadRequest();
        return  articleReadTracker;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public ArticleReadTracker startArticleReadTracker(String articleUrl, String articleId) {
        if (articleUrl == null || articleUrl.isEmpty()) {
            throw new ContentInsightsException(ContentInsightsException.BAD_DATA);
        }

        // Reset previously started trackers if exists
        flushArticle();
        articleReadTracker = new WorkingArticleReadTracker(
            articleUrl, articleId, domainId,
            readTrackingRequest, tokenProvider
        );

        articleReadTracker.sendArticleLoadRequest();
        return  articleReadTracker;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void flushArticle() {
        if (articleReadTracker != null) {
            articleReadTracker.setPageLoaded(false);
            articleReadTracker = null;
        }

        if (attentionTimeTracker != null) {
            attentionTimeTracker.pauseAttention();
            attentionTimeTracker = null;
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void setNewScrollPosition(int position) {
        attentionTimeTracker.setNewScrollPosition(position);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    @RequiresApi(23)
    public void trackScrolling(ScrollView scrollView) {
        attentionTimeTracker.trackScrolling(scrollView);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void trackScrolling(WebView webView) {
        attentionTimeTracker.trackScrolling(webView);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void startAttention() {
        attentionTimeTracker.startAttention();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void pauseAttention() {
        attentionTimeTracker.pauseAttention();
    }

}
