package com.contentinsights.sdk;

import android.content.Context;
import android.test.mock.MockContext;

import com.contentinsights.sdk.helpers.Constants;
import com.contentinsights.sdk.api.SdkFeatures;

import static org.junit.Assert.*;
import org.junit.Test;

public class ContentInsightsConfigTest {

    @Test
    public void basicConfig() {
        ContentInsightsConfig.Builder builder = new ContentInsightsConfig.Builder(mockContext(), "test-domain");
        ContentInsightsConfig config = builder.build();

        assertEquals(config.getDomainId(), "test-domain");
        assertEquals(config.getSessionTimeoutInSeconds(), ContentInsightsConfig.DEFAULT_TIMEOUT);
        assertEquals(config.getConfiguredSdkFeatures(), SdkFeatures.API_CALLING);
    }

    @Test
    public void configWithCustomTimeout() {
        ContentInsightsConfig config =
                new ContentInsightsConfig.Builder(new MockContext(), "test-domain")
                        .sessionTimeoutInSeconds(3600)
                        .build();

        assertEquals(config.getDomainId(), "test-domain");
        assertEquals(config.getSessionTimeoutInSeconds(), 3600);
        assertEquals(config.getConfiguredSdkFeatures(), SdkFeatures.API_CALLING);
    }

    @Test
    public void configWithOverridingCustomTimeout() {
        ContentInsightsConfig config =
                new ContentInsightsConfig.Builder(new MockContext(), "test-domain")
                        .sessionTimeoutInSeconds(1000)
                        .sessionTimeoutInSeconds(2000)
                        .sessionTimeoutInSeconds(3000)
                        .build();

        assertEquals(config.getDomainId(), "test-domain");
        assertEquals(config.getSessionTimeoutInSeconds(), 3000);
        assertEquals(config.getConfiguredSdkFeatures(), SdkFeatures.API_CALLING);
    }


    @Test
    public void debugModeWithPredefinedTimeout() {
        ContentInsightsConfig config =
                new ContentInsightsConfig.Builder(new MockContext(), "test-domain")
                        .sessionTimeoutInSeconds(2400)
                        .noBackendRequests()
                        .build();

        assertEquals(config.getDomainId(), "test-domain");
        assertEquals(config.getSessionTimeoutInSeconds(), 2400);
        assertEquals(config.getConfiguredSdkFeatures(), SdkFeatures.NOTHING);
    }

    @Test
    public void debugModeWithOverridingTimeout() {
        ContentInsightsConfig config =
                new ContentInsightsConfig.Builder(new MockContext(), "test-domain")
                        .sessionTimeoutInSeconds(3000)
                        .noBackendRequests()
                        .sessionTimeoutInSeconds(6000)
                        .build();

        assertEquals(config.getDomainId(), "test-domain");
        assertEquals(config.getSessionTimeoutInSeconds(), 6000);
        assertEquals(config.getConfiguredSdkFeatures(), SdkFeatures.NOTHING);
    }

    @Test
    public void debugModeWithLoggingRequests() {
        ContentInsightsConfig config =
                new ContentInsightsConfig.Builder(new MockContext(), "test-domain")
                        .sessionTimeoutInSeconds(3000)
                        .noBackendRequests()
                        .sessionTimeoutInSeconds(6000)
                        .doLog()
                        .build();

        assertEquals(config.getDomainId(), "test-domain");
        assertEquals(config.getSessionTimeoutInSeconds(), 6000);
        assertEquals(config.getConfiguredSdkFeatures(), SdkFeatures.LOGGING);
    }

    @Test
    public void bothLoggingAndApiCalling() {
        ContentInsightsConfig config =
                new ContentInsightsConfig.Builder(new MockContext(), "test-domain")
                        .sessionTimeoutInSeconds(3000)
                        .doLog()
                        .build();

        assertEquals(config.getDomainId(), "test-domain");
        assertEquals(config.getSessionTimeoutInSeconds(), 3000);
        assertEquals(config.getConfiguredSdkFeatures(), SdkFeatures.API_AND_LOGGING);
        assertEquals(config.getApiUrl(), Constants.API_ROOT);
    }

    private Context mockContext() {
        return new MockContext();
    }
}
